/*
 * AxiliceInitView.java
 *
 * $Id: AxiliceInitView.java,v 1.4 2009-10-27 13:09:23 mario Exp $
 */
package com.imagos.axilice.client.gui;

import java.io.InputStream;
import java.net.URL;

import com.imagos.axilice.client.ClientContext;
import com.imagos.axilice.core.InterfaceAxilice;
import com.imagos.javaglade.Gui;
import com.imagos.javaglade.JavaGlade;
import com.imagos.javaglade.JavaGladeException;
import com.imagos.javaglade.JavaGladeUtil;
import com.imagos.javaglade.handlers.GenericHandler;
import com.imagos.javaglade.toolkit.swt.SwtToolkit;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

/**
 * Helper class that implements the Axilice initializing GUI. 
 * It completes the glade file specification to add extra features.
 * It also implements the event handlers for the Axilice initialing screens 
 * and dialogs.
 * <br/>
 * Responsibilities:
 * <ul>
 * 	<li>Definition and inicialization of the Axilice init GUI. </li>
 *  <li>Event management for the Axilice init windows.</li>
 * </ul>
 * <br/>
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
public class AxiliceInitView extends GenericHandler {
	/** Unique instance of the AxiliceInitView class. Singleton pattern */
	private static AxiliceInitView instance;

	/** Returns the unique instance of the AxiliceInitView class.
	 * Implements the singleton dessign pattern.
	 * @return AxiliceInitView, unique instance of AxiliceInitView.
	 */
	public static final AxiliceInitView getInstance() {
		if (instance == null)
			instance = new AxiliceInitView();
		return instance;
	}
	
	/**
	 * Private default constructor por AxiliceInitView objects.
	 */
	private AxiliceInitView() {
		ClientContext.getInstance().getLogger().
			info("Building the initializing view of the application...");
		
		// Init the logging system for the JavaGlade module
		ClientContext.getInstance().getLogger().
			debug("Initializing the logging system of the JavaGlade. Configuration file found at: " + 
					this.getClass().getResource("/com/imagos/javaglade/log4j.xml").getPath());
		JavaGladeUtil.getInstance().initLogger(AxiliceGUIBuilder.getInstance(), "logs", "/com/imagos/javaglade/log4j.xml");
	}

	/**
	 * Initializes the Axilice Init Viewer engine and builds
	 * the initializing window of the application. Searches automatically for
	 * the glade interface file with the application GUI configuration.
	 */
	public void buildInitView() {
		// Build the main window of the Initializing window
		try {
			ClientContext.getInstance().getLogger().
				info(String.format(InterfaceAxilice.AXILICE_LOG_RESOURCESPATH, ClientContext.getInstance().getResourceDir()));
		
			this.gui = AxiliceGUIBuilder.getInstance().buildGUI(InterfaceAxilice.AXILICE_INIT_GLADE_NAME, InterfaceAxilice.AXILICE_INIT_GLADE_PATH);
		}
		catch (JavaGladeException jEx) {
			ClientContext.getInstance().getLogger().
				error("Error constructing the JavaGlade from the glade init screen file!", jEx);
		}
		catch (Exception ex) {
			ClientContext.getInstance().getLogger().
				error("Error constructing the JavaGlade from the glade init screen file!", ex);
		}
		
		if (this.gui != null) {
			// Set the maximum size of the progress bar
			ProgressBar progressBar = (ProgressBar)this.gui.getWidget(InterfaceAxilice.AXILICE_INIT_PROGRESSBAR_NAME);
			progressBar.setMaximum(100);
			progressBar.setMinimum(0);
			
			Shell initWindow = this.gui.getWindow(InterfaceAxilice.AXILICE_INIT_WINDOW_NAME).getShell();
			initWindow.setSize(520, 350);
			initWindow.setMinimumSize(520, 350);
			
			Monitor primary = SwtToolkit.getDisplay().getPrimaryMonitor ();
			Rectangle bounds = primary.getBounds ();
			Rectangle rect = initWindow.getBounds ();
			System.out.println("MONITOR BOUNDS: " + bounds.width + ", " + bounds.height + " px");
			System.out.println("WINDOW BOUNDS: " + rect.width + ", " + rect.height + " px");
			
			int x = bounds.x + (bounds.width - rect.width) / 2;
			int y = bounds.y + (bounds.height - rect.height) / 2;
			System.out.println("WINDOW LOCATION: " + x + ", " + y + " px");
			initWindow.setLocation (x, y);
		}
		else {
			AxiliceGUIBuilder.getInstance().showConfirmationDialog(null, 
					"Error building initializing GUI. The application will exit now!", 
					"Error in initializing", SWT.ICON_ERROR, SWT.OK);
			System.exit(-1);
		}
	}
	
	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: AxiliceInitView.java,v 1.4 2009-10-27 13:09:23 mario Exp $";
	}
}