/*
 * GUIObject.java
 *
 * $Id: GUIObject.java,v 1.1 2008-03-18 12:58:46 mario Exp $
 *
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * 
 */
package com.imagos.axilice.client.gui;

import com.imagos.axilice.core.util.ResultMessage;

/**
 * Public interface that defines the base event handlers for any GUI object.
 * The base event handlers are:
 * <li>
 * 	<ul> addAction: Event handler for insert operation (prepare the system
 * 	to add a new object).</ul>
 *  <ul> modAction: Event handler for update operation (prepare the system
 *  to update an existent object).</ul>
 *  <ul> delAction: Event handler for delete operation (deletes the object).</ul>
 *  <ul> showAction: Event handler for show operation (shows all the data
 *  of the object to the user).</ul>
 *  <ul> queryAction: Event handler for search operation (looks for the
 *  object in the system).</ul>
 *  <ul> acceptAction: Event handler for accept operation (accepts the changes 
 *  made to the object).</ul>
 *  <ul> cancelAction: Event handler for cancel operation (discards the changes 
 *  made to the object).</ul>
 * </li>
 */
public interface GUIObject {
	public ResultMessage addAction();
	public ResultMessage modAction();
	public ResultMessage delAction();
	public ResultMessage showAction();
	public ResultMessage queryAction();
	public ResultMessage acceptAction();
	public ResultMessage cancelAction();
}
