/*
 * BarcodeTypeOT.java
 *
 * $Id: BarcodeTypeOT.java,v 1.5 2009-03-02 15:31:23 mario Exp $
 */
package com.imagos.axilice.core.common;

import java.io.Serializable;

import com.imagos.axilice.core.InterfaceAxilice;
import com.imagos.axilice.core.warehouse.InterfaceWarehouse;

/**
 * Store class to hold the data of a barcode type.
 * <br/>
 * Responsibilities:
 * <li>
 * 	<ul> Store of barcode type data for exchanging data among presentation, 
 * business and data access layers. </ul>
 * </li>.
 * <br/>
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
public class BarcodeTypeOT extends BaseVO implements Serializable {
	/**
	 * Internal id of the barcode type.
	 */
	private long idBarcodeType;
	/**
	 * Description of the barcode type.
	 */
	private String description;
	/**
	 * Version of the barcode type used by Hibernate for optimistic concurrency control.
	 */
	private Integer version = new Integer(0);
	
	private static final long serialVersionUID = 10000002;
	
	/**
	 * Default BarcodeTypeOT constructor.
	 */
	public BarcodeTypeOT() {
	}
	
	/**
	 * Gets the version of the barcode type used by Hibernate for optimistic concurrency control.
	 * @return Integer, version of the group.
	 */
	public Integer getVersion() {
		return version;
	}
	/**
	 * Sets up the version of the barcode type used by Hibernate for optimistic concurrency control.
	 * @param version Integer, version of the barcode type.
	 */
	public void setVersion(Integer version) {
		this.version = version;
	}

	/**
	 * Gets the description of the barcode type.
	 * @return String, description of the barcode type.
	 */
	public String getDescription() {
		return description;
	}
	/**
	 * Sets up the description of the barcode type.
	 * @param description String, description of the barcode type.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Gets the identifier of the barcode type.
	 * @return String, identifier of the barcode type.
	 */
	public long getIdBarcodeType() {
		return idBarcodeType;
	}
	/**
	 * Sets up the identifier of the barcode type.
	 * @param idGroup String, identifier of the barcode type.
	 */
	public void setIdBarcodeType(long idBarcodeType) {
		this.idBarcodeType = idBarcodeType;
	}
	
	/**
	 * Implementation of the equals method provided by the serializable interface.
	 */
	public boolean equals(Object other) {
        if (this == other) 
        	return true;
        if ( !(other instanceof BarcodeTypeOT) ) 
        	return false;

        final BarcodeTypeOT barcodeType = (BarcodeTypeOT) other;

        if ( barcodeType.getDescription() == null )
        	return this.getDescription() == null;
        else if ( !barcodeType.getDescription().equals( this.getDescription() ) ) 
        	return false;

        return true;
    }

	/**
	 * Implementation of the hashCode method provided by the serializable interface.
	 */
    public int hashCode() {
        int result;
        if (this == null || this.getDescription() == null)
        	result = 0;
        else
        	result = this.getDescription().hashCode();
        return result;
    }
    
	/**
	 * Implementation of the toString() method to show the data of the ExtFieldOT
	 * formatted.
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		try {
			result.append("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
			result.append("\nID: ");
			result.append(this.idBarcodeType);
			result.append("\nDESCRIPTION: ");
			result.append(this.description);
			result.append("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
		}
		catch (Exception ex) {
			System.out.println("Error formatting BarcodeTypeOT!");
			ex.printStackTrace();
			return null;
		}
		
		return result.toString();
	}
    
	@Override
	public boolean equals(Long id) {
		return id != null && this.idBarcodeType == id.longValue();
	}
	
	@Override
	public long getInternalId() {
		return this.idBarcodeType;
	}

	@Override
	public String getInternalValue() {
		return this.description;
	}
	
	@Override
	public String getEntityId() {
		return InterfaceAxilice.AXILICE_ENTITY_BARCODETYPE_ID;
	}

	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: BarcodeTypeOT.java,v 1.5 2009-03-02 15:31:23 mario Exp $";
	}	
}
