package com.imagos.axilice.core.common;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.imagos.axilice.core.AxiliceException;
import com.imagos.axilice.core.partners.AddressOT;
import com.imagos.axilice.core.util.OperationResult;
import com.imagos.axilice.core.util.ResultMessage;

import com.imagos.axilice.core.common.BarcodeTypeOT;
import com.imagos.axilice.core.common.VatTypeOT;
import com.imagos.axilice.core.extTables.ExtFieldOT;
import com.imagos.axilice.core.extTables.ExtTableDAO;
import com.imagos.axilice.core.extTables.ExtTableException;
import com.imagos.axilice.core.extTables.ExtTableOT;
import com.imagos.axilice.server.util.db.CriteriaBean;
import org.hibernate.criterion.Criterion;

public interface CommonManager extends Remote {
	public List queryBarcodeTypes(String fieldId, String value) throws RemoteException, AxiliceException;
	public OperationResult delBarcodeType(Long idBarcodeType) throws RemoteException, AxiliceException;
	public OperationResult delBarcodeTypes(ArrayList<Long> lSelBarcodeTypes) throws RemoteException, AxiliceException;
	public OperationResult saveBarcodeType(BarcodeTypeOT barcodeType) throws RemoteException, AxiliceException;
	public BarcodeTypeOT getBarcodeType(Long idBarcodeType) throws RemoteException, AxiliceException;
	
	public List queryVatTypes(String fieldId, String value) throws RemoteException, AxiliceException;
	public OperationResult delVatType(Long idVatType) throws RemoteException, AxiliceException;
	public OperationResult delVatTypes(ArrayList<Long> lSelVatTypes) throws RemoteException, AxiliceException;
	public OperationResult saveVatType(VatTypeOT vatType) throws RemoteException, AxiliceException;
	public VatTypeOT getVatType(Long idVatType) throws RemoteException, AxiliceException;
	/*
	public List queryAddresses(String fieldId, String value) throws RemoteException, AxiliceException;
	public OperationResult delAddress(Long idAddress) throws RemoteException, AxiliceException;
	public OperationResult delAddresses(ArrayList<Long> lSelAddresses) throws RemoteException, AxiliceException;
	public OperationResult saveAddress(AddressOT address) throws RemoteException, AxiliceException;
	public AddressOT getAddress(Long idAddress) throws RemoteException, AxiliceException;
	*/
	/**
	 * Return the extension table definition, including the list of all the
	 * extension fields defined for the table.
	 * @param codExtTable String, external code of the extension table.
	 * @return ExtTableOT, extension table definition.
	 * @throws ExtTableException in case any error take place during the
	 * query operation.
	 */
	public ExtTableOT getExtensionTableDefinition(String codExtTable) throws RemoteException, ExtTableException;

	
	/**
	 * Return all the values stored for an extended item. Include all the fields
	 * defined for the extension table and their correspondent value (if anyone 
	 * has been defined).
	 * @param idValue Long, internal id of the extended item. 
	 * @param codExtTable String, external code of the extension table.
	 * @return Map, set of all the extension fields and their values for the 
	 * extended item.
	 * @throws ExtTableException, in case any error take place during the query 
	 * operation.
	 */
	public Map getExtendedItemValues(Long idValue, String codExtTable) throws RemoteException, ExtTableException;
	
	/**
	 * Update the value of all the extension fields defined for an extended item.
	 * @param lExtFields ArrayList, list of all the extension fields whose values
	 * must be updated.
	 * @param idItem long, internal id of the extended item.
	 * @return OperationResult, result of the save extended values operation.
	 * @throws ExtTableException, in case any error take place during
	 * the update operation.
	 */
	public OperationResult saveExtendedItemValues(ArrayList lExtFields, long idItem) throws RemoteException, ExtTableException;
	
	/**
	 * Update the value of all the extension fields defined for an extended item.
	 * @param mExtFields Map, set of all the extension fields whose values
	 * must be updated.
	 * @param idItem long, internal id of the extended item.
	 * @throws ExtTableException, in case any error take place during
	 * the update operation.
	 */
	public void saveExtendedItemValues(Map mExtFields, long idItem) throws RemoteException, ExtTableException;
	
	
	
	
	public List<BaseVO> queryEntities(String fieldId, String value, Class myClass) throws AxiliceException, RemoteException;
	public BaseVO getEntity(Long id, Class myClass) throws RemoteException, AxiliceException;
	//public OperationResult delEntity(Long id, Class myClass) throws RemoteException, AxiliceException;
	public OperationResult saveEntity(BaseVO entity) throws RemoteException, AxiliceException;
	public OperationResult delEntities(ArrayList<Long> lSel, Class myClass) throws RemoteException, AxiliceException;
	//public OperationResult delEntity(BaseVO entity) throws RemoteException, AxiliceException;
	
	public List<BaseVO> queryEntities(String fieldId, BaseVO value, Class myClass) throws RemoteException, AxiliceException;
	public OperationResult delEntity(BaseVO b) throws RemoteException, AxiliceException;
	public List<BaseVO> queryEntities(String  sql) throws RemoteException, AxiliceException;
	public BaseVO getEntity(String fieldId, String value, Class myClass) throws AxiliceException, RemoteException;
	
	public OperationResult saveImage(String imgName, byte[] contents) throws RemoteException, AxiliceException;
}