/*
 * CountryOT.java
 *
 * $Id: CountryOT.java,v 1.3 2009-06-23 12:54:37 juancarlos Exp $
 */
package com.imagos.axilice.core.common;

import java.io.Serializable;

import com.imagos.axilice.core.InterfaceAxilice;

/**
 * Store class to hold the data of a country.
 * <br/>
 * Responsibilities:
 * <li>
 * 	<ul> Store of country data for exchanging data among presentation, 
 * business and data access layers. </ul>
 * </li>.
 * <br/>
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
public class CountryOT extends BaseVO implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Internal id of the country.
	 */
	private long idCountry;
	
	private String name;
	private FlagOT flag;
	
	/**
	 * Version of the country used by Hibernate for optimistic concurrency control.
	 */
	private Integer version = new Integer(0);

	
	
	
	
	public CountryOT() {
		super();
		// TODO Auto-generated constructor stub
	}
	public FlagOT getFlag() {
		return flag;
	}
	public void setFlag(FlagOT flag) {
		this.flag = flag;
	}

	public long getIdCountry() {
		return idCountry;
	}
	public void setIdCountry(long idCountry) {
		this.idCountry = idCountry;
	}

	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the version of the country used by Hibernate for optimistic concurrency control.
	 * @return Integer, version of the group.
	 */
	public Integer getVersion() {
		return version;
	}
	/**
	 * Sets up the version of the country used by Hibernate for optimistic concurrency control.
	 * @param version Integer, version of the group.
	 */
	public void setVersion(Integer version) {
		this.version = version;
	}
	
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		//result = prime * result + ((flag == null) ? 0 : flag.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj==null)
			return false;
		//if (!super.equals(obj))
		//	return false;
		if (getClass() != obj.getClass())
			return false;
		CountryOT other = (CountryOT) obj;
		if (flag == null) {
			if (other.flag != null)
				return false;
		} else if (!flag.equals(other.flag))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}
	/**
	 * Implementation of the toString() method to show the data of the CountryOT
	 * formatted.
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		try {
			result.append("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
			result.append("\nID: ");
			result.append(this.idCountry);
			result.append("\nNAME: ");
			result.append(this.name);
			result.append("\nFLAG: ");
			result.append(this.flag != null?this.flag.getInternalValue():"NO FLAG");
			
			result.append("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
		}
		catch (Exception ex) {
			System.out.println("Error formatting CountryOT!");
			ex.printStackTrace();
			return null;
		}
		
		return result.toString();
	}
	
	@Override
	public boolean equals(Long id) {
		return id != null && this.idCountry == id.longValue();
	}
	
	@Override
	public long getInternalId() {
		return this.idCountry;
	}

	@Override
	public String getInternalValue() {
		return this.name;
	}
	
	@Override
	public String getEntityId() {
		return InterfaceAxilice.AXILICE_ENTITY_COUNTRY_ID;
	}
	
	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: CountryOT.java,v 1.3 2009-06-23 12:54:37 juancarlos Exp $";
	}
}
