/*
 * ModuleEntityResolver.java
 *
 * $Id: ModuleEntityResolver.java,v 1.1 2008-03-18 12:58:45 mario Exp $
 */
package com.imagos.axilice.core.installer;

import java.io.IOException;
import java.net.URL;

import com.imagos.axilice.core.InterfaceAxilice;
import com.imagos.axilice.core.util.LoggerFactory;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
* This class implements a SAX Entity Resolver to update the DTD used in the Module
* Config XML file Parsing to the local version. That way the application can work 
* even with no Internet connection and the parsing is faster.
* <br/>
* Responsibilities:
* <ul>
*  <li>Translate the web URL of the DTD to a local URI.</li>
* </ul>
* <br/>
* @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
public class ModuleEntityResolver implements EntityResolver {
	/**
	 * Allow the application to resolve external entities. Its the implementation
	 * of the resolveEntity method of the EntityResolver interface. In this method
	 * the DTD external entity will be redirected to its local version.
	 * @param publicId String, The public identifier of the external entity being referenced, 
	 * or null if none was supplied.
	 * @param systemId String, The system identifier of the external entity being referenced.
	 * @return InputSource An InputSource object describing the new input source, or null to request that the 
	 * parser open a regular URI connection to the system identifier.
	 */
	public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
		if (systemId != null && systemId.contains("module.dtd")) {
			try {
				URL url = this.getClass().getResource("/com/imagos/axilice/core/installer/module.dtd");
				LoggerFactory.getLogger(InterfaceAxilice.AXILICE_LOG_CATEGORY).
					info(String.format(InterfaceAxilice.AXILICE_LOG_RESOLVE_DTD, (url==null?"null":url.getPath())));
				return new InputSource(this.getClass().getResourceAsStream("/com/imagos/axilice/core/installer/module.dtd"));
			}
			catch (Exception ex) {
				LoggerFactory.getLogger(InterfaceAxilice.AXILICE_LOG_CATEGORY).
					error(InterfaceAxilice.AXILICE_LOG_RESOLVE_DTD_ERROR, ex);
				return null;
			}
		}
		else {
			return null;
		}
	}

	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: ModuleEntityResolver.java,v 1.1 2008-03-18 12:58:45 mario Exp $";
	}
}