/*
 * ResourceException.java
 *
 * $Id: ResourceException.java,v 1.1 2008-03-18 12:58:45 mario Exp $
 *
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * 
 */
package com.imagos.axilice.core.installer;

/**
 * This class encapsulates and holds all the errors that might have taken
 * place during any of the application resources operations.
 * <br/>
 * Responsibilities:
 * <li>
 * 	<ul> Encapsulate and pass on the error messages to the upper layers. </ul>
 * </li>
 */
public class ResourceException extends Exception {
	private static final long serialVersionUID = 10000009;
	
	/**
	 * Code of the resource involved in the operation that
	 * originated the error.
	 */
	private String codResource;
	/**
	 * Path of the resource involved in the operation that
	 * originated the error.
	 */
	private String pathResource;
	/**
	 * Internal error code to support error localization and classification.
	 */
	private int errCode;
	/**
	 * Error message string that is showed to the end user.
	 */
	private String errMessage;

	public ResourceException(int errCode) {
		super();
		this.errCode = errCode;
		this.codResource = null;
		this.pathResource = null;
		
		this.errMessage = this.getErrorMessage(this.errCode);
	}
	
	public ResourceException(int errCode, Throwable cause) {
		super(cause);
		this.errCode = errCode;
		this.codResource = null;
		
		this.errMessage = this.getErrorMessage(this.errCode);
	}
	
	public ResourceException(String codResource, int errCode) {
		super();
		this.errCode = errCode;
		this.codResource = codResource;
		this.pathResource = null;
		
		this.errMessage = this.getErrorMessage(this.errCode);
	}
	
	public ResourceException(String codResource, int errCode, Throwable cause) {
		super(cause);
		this.errCode = errCode;
		this.codResource = codResource;
		this.pathResource = null;
		
		this.errMessage = this.getErrorMessage(this.errCode);
	}
	
	public ResourceException(String codResource, String pathResource, int errCode) {
		super();
		this.errCode = errCode;
		this.codResource = codResource;
		this.pathResource = pathResource;
		
		this.errMessage = this.getErrorMessage(this.errCode);
	}
	
	public ResourceException(String codResource, String pathResource, int errCode, Throwable cause) {
		super(cause);
		this.errCode = errCode;
		this.codResource = codResource;
		this.pathResource = pathResource;
		
		this.errMessage = this.getErrorMessage(this.errCode);
	}
	
	public String getCodResource() {
		return codResource;
	}

	public void setCodResource(String codResource) {
		this.codResource = codResource;
	}
	
	public String getPathResource() {
		return pathResource;
	}

	public void setPathResource(String pathResource) {
		this.pathResource = pathResource;
	}

	public void setErrMessage(String errMessage) {
		this.errMessage = errMessage;
	}
	
	public String getErrMessage() {
		return errMessage;
	}
	
	public String getMessage() {
		return this.toString();
	}

	public String toString() {
		StringBuffer msg = new StringBuffer();
		msg.append("¡¡¡¡  ERROR ");
		msg.append(this.errCode);
		msg.append("  !!!!    ");
		if (this.codResource != null && this.codResource.length() > 0) {
			msg.append("[");
			msg.append(this.codResource);
			msg.append("]");
		}
		
		if (this.pathResource != null && this.pathResource.length() > 0) {
			msg.append("(PATH=");
			msg.append(this.pathResource);
			msg.append(")");
		}
		
		msg.append("\t");
		msg.append(this.errMessage);
		
		return msg.toString();
	}

	public int getErrCode() {
		return errCode;
	}

	public void setErrCode(int errCode) {
		this.errCode = errCode;
	}
	
	/**
	 * Get the localized error message corresponding to the error code.
	 * @param errCode int, error code.
	 * @return String, localized error message corresponding to the error code.
	 */
	private String getErrorMessage(int errCode) {
		StringBuffer msg = new StringBuffer();
		
		// TO_DO: Obtener el correspondiente mensaje para ese codigo de error
		return msg.toString();
	}
	
	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: ResourceException.java,v 1.1 2008-03-18 12:58:45 mario Exp $";
	}
}

