/*
 * WorkcenterInfo.java
 *
 * $Id: WorkcenterInfo.java,v 1.1 2008-03-18 12:58:46 mario Exp $
 */
package com.imagos.axilice.core.traceability.tree;

/**
 * ValueObject with the data of a Workcenter item.
 * <br/>
 * Responsibilities:
 * <ul>
 *  <li>Store the workcenter data.</li>
 * </ul>
 * <br/>
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
public class WorkcenterInfo {
	/** Workcenter code */
	private String codWorkcenter;
	/** Workcenter description */
	private String description;
	
	/**
	 * Default WorkcenterInfo constructor.
	 */
	public WorkcenterInfo() {
		this.setCodWorkcenter(null);
		this.setDescription(null);
	}
	
	/**
	 * WorkcenterInfo constructor that initializes the workcenter values.
	 * @param codWorkcenter
	 * @param description
	 */
	public WorkcenterInfo(String codWorkcenter, String description) {
		this.setCodWorkcenter(codWorkcenter);
		this.setDescription(description);
	}
	
	/**
	 * Gets the workcenter description.
	 * @return String, workcenter description.
	 */
	public String getDescription() {
		return description;
	}
	/**
	 * Sets up the workcenter description.
	 * @param description String, a description.
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	
	/**
	 * Gets the workcenter code.
	 * @return String, workcenter code.
	 */
	public String getCodWorkcenter() {
		return codWorkcenter;
	}
	/**
	 * Sets up the workcenter code.
	 * @param codWorkcenter String, a workcenter code.
	 */
	public void setCodWorkcenter(String codWorkcenter) {
		this.codWorkcenter = codWorkcenter;
	}
	
	/**
	 * Converts the workcenter info to a formatted String.
	 * @return String, formatted String with the workcenter's data.
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append(this.codWorkcenter);
		result.append(":  ");
		result.append(this.description);
		
		return result.toString();
	}
	
	/**
	 * Updates the workcenter data.
	 * @param info WorkcenterInfo, new workcenter data.
	 */
	public void update(WorkcenterInfo info) {
		this.setCodWorkcenter(info.getCodWorkcenter());
		this.setDescription(info.getDescription());
	}
	
	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: WorkcenterInfo.java,v 1.1 2008-03-18 12:58:46 mario Exp $";
	}
}
