/*
 * ProductGroupOT.java
 *
 * $Id: ProductGroupOT.java,v 1.3 2009-03-02 15:31:23 mario Exp $
 */
package com.imagos.axilice.core.warehouse;

import java.io.Serializable;

import com.imagos.axilice.core.common.BaseVO;
import com.imagos.axilice.core.extTables.ExtTableOT;

/**
 * Store class to hold the data of a product group.
 * <br/>
 * Responsibilities:
 * <li>
 * 	<ul> Store of product group data for exchanging data among presentation, 
 * business and data access layers. </ul>
 * </li>.
 * <br/>
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
public class ProductGroupOT extends BaseVO implements Serializable {
	/**
	 * Internal id of the product group.
	 */
	private long idGroup;
	/**
	 * Description of the product group.
	 */
	private String description;
	/**
	 * Extension table defined for the group of products.
	 */
	private ExtTableOT extTable;
	/**
	 * Version of the product group used by Hibernate for optimistic concurrency control.
	 */
	private Integer version = new Integer(0);
	
	private static final long serialVersionUID = 10000002;
	
	/**
	 * Default ProductGroupOT constructor.
	 */
	public ProductGroupOT() {
	}
	
	/**
	 * Gets the version of the product group used by Hibernate for optimistic concurrency control.
	 * @return Integer, version of the group.
	 */
	public Integer getVersion() {
		return version;
	}
	/**
	 * Sets up the version of the product group used by Hibernate for optimistic concurrency control.
	 * @param version Integer, version of the group.
	 */
	public void setVersion(Integer version) {
		this.version = version;
	}

	/**
	 * Gets the description of the product group.
	 * @return String, description of the group.
	 */
	public String getDescription() {
		return description;
	}
	/**
	 * Sets up the description of the product group.
	 * @param description String, description of the group.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Gets the identifier of the product group.
	 * @return String, identifier of the group.
	 */
	public long getIdGroup() {
		return idGroup;
	}
	/**
	 * Sets up the identifier of the product group.
	 * @param idGroup String, identifier of the group.
	 */
	public void setIdGroup(long idGroup) {
		this.idGroup = idGroup;
	}
	
	/**
	 * Gets the extension table for the product group.
	 * @return ExtTableOT, extension table of the group.
	 */
	public ExtTableOT getExtTable() {
		return extTable;
	}
	/**
	 * Sets up the extension table for the product group.
	 * @param extTable ExtTableOT, extension table of the group.
	 */
	public void setExtTable(ExtTableOT extTable) {
		this.extTable = extTable;
	}

	/**
	 * Implementation of the equals method provided by the serializable interface.
	 */
	public boolean equals(Object other) {
        if (this == other) 
        	return true;
        if ( !(other instanceof ProductGroupOT) ) 
        	return false;

        final ProductGroupOT group = (ProductGroupOT) other;

        if ( group.getDescription() == null )
        	return this.getDescription() == null;
        else if ( !group.getDescription().equals( this.getDescription() ) ) 
        	return false;

        return true;
    }

	/**
	 * Implementation of the hashCode method provided by the serializable interface.
	 */
    public int hashCode() {
        int result;
        if (this == null || this.getDescription() == null)
        	result = 0;
        else
        	result = this.getDescription().hashCode();
        return result;
    }
    
	/**
	 * Implementation of the toString() method to show the data of the ExtFieldOT
	 * formatted.
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		try {
			result.append("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
			result.append("\nID: ");
			result.append(this.idGroup);
			result.append("\nDESCRIPTION: ");
			result.append(this.description);
			result.append("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
		}
		catch (Exception ex) {
			System.out.println("Error formatting ProductGroupOT!");
			ex.printStackTrace();
			return null;
		}
		
		return result.toString();
	}
	
	@Override
	public boolean equals(Long id) {
		return id != null && this.idGroup == id.longValue();
	}
	
	@Override
	public long getInternalId() {
		return this.idGroup;
	}

	@Override
	public String getInternalValue() {
		return this.description;
	}
	
	@Override
	public String getEntityId() {
		return InterfaceWarehouse.AXILICE_ENTITY_PRODUCTGROUP_ID;
	}

	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: ProductGroupOT.java,v 1.3 2009-03-02 15:31:23 mario Exp $";
	}	
}
