package com.imagos.axilice.server.common;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import com.imagos.axilice.core.InterfaceAxilice;
import com.imagos.axilice.core.common.CommonManager;
import com.imagos.axilice.core.management.InterfaceManagement;
import com.imagos.axilice.core.ssl.RMISSLClientSocketFactory;
import com.imagos.axilice.core.ssl.RMISSLServerSocketFactory;
import com.imagos.axilice.core.util.LoggerFactory;
import com.imagos.axilice.server.ServerContext;

public class CommonManagerBuilder {
	/**
	 * Initializes the common manager remote object. Makes the registration and
	 * binding of the remote CommonManager service.  
	 */
	public static void initCommonManager() {
		if (System.getSecurityManager() == null) {
            System.setSecurityManager(new SecurityManager());
        }
        try {
            String name = InterfaceAxilice.JNDI_COMMON_MANAGER_EXPORT_NAME;
            CommonManagerImpl manager = new CommonManagerImpl();
            
            UnicastRemoteObject.unexportObject(manager, true);
            System.out.println("CommonManager unexported");


            /**
             * Export the CommonManager object to listen in the assigned port
             */
            CommonManager stub =
                (CommonManager) UnicastRemoteObject.exportObject(manager, InterfaceAxilice.RMI_COMMON_MANAGER_EXPORT_PORT, 
                		new RMISSLClientSocketFactory(),
               		 	new RMISSLServerSocketFactory());
            
            System.out.println("CommonManager exported");

            /*
    	     * Create a registry and bind stub in registry.
    	     */
//    	    LocateRegistry.createRegistry(InterfaceAxilice.RMI_REGISTRY_PORT);
    	    Registry registry = LocateRegistry.getRegistry(InterfaceAxilice.RMI_REGISTRY_PORT);
    	    
            System.out.println("Registry located");
            registry.rebind(name, stub);
            System.out.println("CommonManager bound");
        } catch (Exception e) {
            System.err.println("CommonManager exception:");
            e.printStackTrace();
        }
        // Init the logger service
		System.out.println("LOGGING CONFIGURATION PATH= " + ServerContext.getInstance().getClass().getResource(InterfaceAxilice.COMMON_LOG_CONFIG_FILE).getPath());
		LoggerFactory.initLogger(
				InterfaceAxilice.AXILICE_LOG_DIR,
				ServerContext.getInstance().getClass().getResourceAsStream(InterfaceAxilice.COMMON_LOG_CONFIG_FILE));
	
        
	}
}
