
package com.imagos.axilice.server.common.bl;

import java.util.ArrayList;

import com.imagos.axilice.core.AxiliceException;
import com.imagos.axilice.core.common.BarcodeTypeOT;
import com.imagos.axilice.core.common.BaseVO;
import com.imagos.axilice.core.common.VatTypeOT;
import com.imagos.axilice.core.util.OperationResult;
import com.imagos.axilice.server.ServerContext;
import com.imagos.axilice.server.common.dao.BarcodeTypeDAO;
import com.imagos.axilice.server.common.dao.CommonDAO;
import com.imagos.axilice.server.common.dao.VatTypeDAO;
import com.imagos.axilice.server.util.db.CriteriaBean;

/**
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * 
 * <br>
 * <br>
 *         This program is free software: you can redistribute it and/or modify
 *         it under the terms of the GNU General Public License as published by
 *         the Free Software Foundation, either version 3 of the License, or (at
 *         your option) any later version. <br/>
 *         This program is distributed in the hope that it will be useful, but
 *         WITHOUT ANY WARRANTY 
 *         <br>without even the implied warranty of
 *         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *         General Public License for more details. <br/>
 *         You should have received a copy of the GNU General Public License
 *         along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
public class CommonBLL {
	/** Unique instance of the ManagementBLL class. Singleton pattern */
	private static CommonBLL instance;

	/** Returns the unique instance of the ManagementBLL class.
	 * Implements the singleton dessign pattern.
	 * @return ManagementBLL, unique instance of ManagementBLL.
	 */
	public static final CommonBLL getInstance() {
		if (instance == null)
			instance = new CommonBLL();
		return instance;
	}

	/**
	 * Default constructor.
	 */
	private CommonBLL() {

	}

	public ArrayList queryBarcodeTypes(ArrayList <CriteriaBean> lCriteria) throws AxiliceException {
		// Call the DAO to query the database and get the barcode types.
		BarcodeTypeDAO dao = new BarcodeTypeDAO();
		ArrayList lBarcodeTypes = dao.getBarcodeTypes(lCriteria);
		
		ServerContext.getInstance().getLogger().info("List of barcode types: " + lBarcodeTypes);
		
		return lBarcodeTypes;
	}
	
	public OperationResult delBarcodeType(Long id) throws AxiliceException {
		// Call the DAO to remove the barcode type from the database.
		BarcodeTypeDAO dao = new BarcodeTypeDAO();
		return dao.delBarcodeType(id);
	}
	
	public OperationResult delBarcodeTypes(ArrayList<Long> lSelBarcodeTypes) throws AxiliceException {
		// Call the DAO to remove the barcode types from the database.
		BarcodeTypeDAO dao = new BarcodeTypeDAO();
		return dao.delBarcodeTypes(lSelBarcodeTypes);
	}
	
	public OperationResult saveBarcodeType(BarcodeTypeOT barcodeType) throws AxiliceException {
		// Call the DAO to save the barcode type in the database.
		BarcodeTypeDAO dao = new BarcodeTypeDAO();
		return dao.saveBarcodeType(barcodeType);
	}
	
	public BarcodeTypeOT getBarcodeType(Long id) throws AxiliceException {
		BarcodeTypeDAO dao = new BarcodeTypeDAO();
		return dao.getBarcodeType(id.longValue());
	}
	
	public ArrayList queryVatTypes(ArrayList <CriteriaBean> lCriteria) throws AxiliceException {
		// Call the DAO to query the database and get the barcode types.
		VatTypeDAO dao = new VatTypeDAO();
		ArrayList lVatTypes = dao.getVatTypes(lCriteria);
		
		ServerContext.getInstance().getLogger().info("List of VAT types: " + lVatTypes);
		
		return lVatTypes;
	}
	
	public OperationResult delVatType(Long id) throws AxiliceException {
		// Call the DAO to remove the barcode type from the database.
		VatTypeDAO dao = new VatTypeDAO();
		return dao.delVatType(id);
	}
	
	public OperationResult delVatTypes(ArrayList<Long> lSelVatTypes) throws AxiliceException {
		// Call the DAO to remove the barcode types from the database.
		VatTypeDAO dao = new VatTypeDAO();
		return dao.delVatTypes(lSelVatTypes);
	}
	
	public OperationResult saveVatType(VatTypeOT vatType) throws AxiliceException {
		// Call the DAO to save the VAT type in the database.
		VatTypeDAO dao = new VatTypeDAO();
		return dao.saveVatType(vatType);
	}
	
	public VatTypeOT getVatType(Long id) throws AxiliceException {
		VatTypeDAO dao = new VatTypeDAO();
		return dao.getVatType(id.longValue());
	}
	

	

	public BaseVO getEntity(Long id, Class myClass) throws AxiliceException{
		CommonDAO myDAO=new CommonDAO();
		return myDAO.getEntity(id, myClass);	
		
	}

	
	public ArrayList<BaseVO> queryEntities(ArrayList <CriteriaBean> lCriteria, Class myClass) throws AxiliceException {

		CommonDAO dao = new CommonDAO();
		ArrayList<BaseVO> lEntities = dao.getEntities(lCriteria, myClass);
		return lEntities;
	}
	

	public ArrayList<BaseVO> queryEntities(String hql) throws AxiliceException{
		CommonDAO dao = new CommonDAO();
		ArrayList<BaseVO> lEntities = dao.getEntities(hql);
		return lEntities;
	}
	

		public OperationResult delEntities(ArrayList<Long> lSelected, Class myClass) throws AxiliceException {

			CommonDAO dao = new CommonDAO();
			return dao.delEntities(lSelected, myClass);
		}
		
		public OperationResult saveEntity(BaseVO entity) throws AxiliceException {
			CommonDAO dao = new CommonDAO();
			return dao.saveEntity(entity);
		}
		
	


	public OperationResult delEntity(BaseVO b) throws AxiliceException{

		CommonDAO dao = new CommonDAO();
		return dao.delEntity(b);
	}
}