package com.imagos.axilice.server.management;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import com.imagos.axilice.core.InterfaceAxilice;
import com.imagos.axilice.core.management.InterfaceManagement;
import com.imagos.axilice.core.management.ManagementManager;
import com.imagos.axilice.core.ssl.RMISSLClientSocketFactory;
import com.imagos.axilice.core.ssl.RMISSLServerSocketFactory;
import com.imagos.axilice.core.util.LoggerFactory;
import com.imagos.axilice.core.warehouse.InterfaceWarehouse;
import com.imagos.axilice.server.ServerContext;

public class ManagementManagerBuilder {
	/**
	 * Initializes the common manager remote object. Makes the registration and
	 * binding of the remote ManagementManager service.  
	 */
	public static void initManagementManager() {
		if (System.getSecurityManager() == null) {
            System.setSecurityManager(new SecurityManager());
        }
        try {
            String name = InterfaceManagement.JNDI_MANAGEMENT_MANAGER_EXPORT_NAME;
            ManagementManagerImpl manager = new ManagementManagerImpl();
            
            UnicastRemoteObject.unexportObject(manager, true);
            System.out.println("ManagementManager unexported");


            /**
             * Export the CommonManager object to listen in the assigned port
             */
            ManagementManager stub =
                (ManagementManager) UnicastRemoteObject.exportObject(manager, InterfaceManagement.RMI_MANAGEMENT_MANAGER_EXPORT_PORT, 
                		new RMISSLClientSocketFactory(),
               		 	new RMISSLServerSocketFactory());
            
            System.out.println("ManagementManager exported");

            /*
    	     * Create a registry and bind stub in registry.
    	     */
//    	    LocateRegistry.createRegistry(InterfaceAxilice.RMI_REGISTRY_PORT);
    	    Registry registry = LocateRegistry.getRegistry(InterfaceAxilice.RMI_REGISTRY_PORT);
    	    
            System.out.println("Registry located");
            registry.rebind(name, stub);
            System.out.println("ManagementManager bound");
        } catch (Exception e) {
            System.err.println("ManagementManager exception:");
            e.printStackTrace();
        }
        // Init the logger service
		System.out.println("LOGGING CONFIGURATION PATH= " + ServerContext.getInstance().getClass().getResource(InterfaceManagement.MANAGEMENT_LOG_CONFIG_FILE).getPath());
		LoggerFactory.initLogger(
				InterfaceAxilice.AXILICE_LOG_DIR,
				ServerContext.getInstance().getClass().getResourceAsStream(InterfaceManagement.MANAGEMENT_LOG_CONFIG_FILE));
	
        
        
	}
}
